/*
 * Copyright 2007 by Kappich Systemberatung, Aachen
 * Copyright 2005 by Kappich+Kni Systemberatung Aachen (K2S)
 * 
 * This file is part of de.bsvrz.dav.daf.
 * 
 * de.bsvrz.dav.daf is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.dav.daf is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with de.bsvrz.dav.daf; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package de.bsvrz.dav.daf.main.config;

import java.util.*;

/**
 * Schnittstelle zum Zugriff auf die Eigenschaften eines Konfigurationsobjektes. Konfigurationsobjekte haben neben den Eigenschaften aller System-Objekte eine
 * verantwortliche Instanz (Zustndiger), eine Version der Konfiguration ab der das Objekt gltig ist und eine Version der Konfiguration ab der das Objekt nicht
 * mehr gltig ist.<BR/> Die verantwortliche Instanz macht eine Aussage darber in welcher Umgebung Konfigurationsnderungen fr das Objekt durchgefhrt werden
 * knnen. Dies ist fr Anwendungsobjekte blicherweise eine Verkehrsrechnerzentrale oder eine Unterzentrale und fr Objekte des Datenmodells eine globale
 * Instanz.<BR/> In einer Konfiguration knnen Konfigurationsobjekte von verschiedenen verantwortlichen Instanzen gespeichert werden. Jede Konfiguration
 * verwaltet fr jede verantwortliche Instanz zu der sie Objekte gespeichert hat eine lokal aktivierte Versionsnummer die nicht unbedingt mit der in der
 * verantwortlichen Instanz aktivierten Versionsnummer bereinstimmen muss. Beispielsweise knnen in der Konfiguration einer VRZ die Konfigurationsobjekte einer
 * UZ bernommen werden. Die VRZ verwaltet dann fr die bernommene Konfiguration eine lokale Versionsnummer.<BR/> Die Versionsnummern ab der ein
 * Konfigurationsobjekt gltig bzw. nicht mehr gltig ist bezieht sich immer auf die lokal aktivierte Versionsnummer der verantwortlichen Instanz des jeweiligen
 * Objekts. Wenn in der VRZ die lokal aktivierte Version der UZ-Konfiguration kleiner als die in der UZ aktivierten Version ist, dann werden z.B. die neusten
 * Objekte der UZ in der VRZ als noch nicht gltig angesehen, obwohl sie in der UZ schon gltig sind.<BR/> Die Schnittstelle bietet auerdem Mglichkeiten, um
 * auf die einem Konfigurationsobjekt zugeordneten benannten Objekt-Mengen zuzugreifen.
 *
 * @author Kappich Systemberatung
 * @version $Revision: 5084 $
 */
public interface ConfigurationObject extends SystemObject {

	/**
	 * Liefert die Versionsnummer der Konfiguration ab der dieses Konfigurationsobjekt gltig ist.
	 *
	 * @return Version ab der das Objekt gltig ist.
	 */
	public short getValidSince();

	/**
	 * Liefert die Versionsnummer der Konfiguration ab der dieses Konfigurationsobjekt nicht mehr gltig ist.
	 *
	 * @return Versionsnummer der Konfiguration ab der dieses Konfigurationsobjekt nicht mehr gltig ist. Wenn das Objekt noch nicht mit der Methode {@link
	 *         SystemObject#invalidate} ungltig gemacht worden ist, dann wird der Wert <code>0</code> zurckgegeben.
	 */
	public short getNotValidSince();

	/**
	 * Liefert eine diesem Konfigurations-Objekt zugeordnete nicht online nderbare Menge zurck.
	 *
	 * @param name Der Name der gewnschten Menge
	 *
	 * @return Gewnschte Konfigurationsmenge oder <code>null</code>, wenn die spezifizierte Menge nicht vorhanden ist.
	 */
	public NonMutableSet getNonMutableSet(String name);

	/**
	 * Liefert eine diesem Konfigurations-Objekt zugeordnete online nderbare Menge zurck.
	 *
	 * @param name Der Name der gewnschten Menge
	 *
	 * @return Gewnschte dynamische Menge oder <code>null</code>, wenn die spezifizierte Menge nicht vorhanden ist.
	 */
	public MutableSet getMutableSet(String name);

	/**
	 * Liefert eine diesem Konfigurations-Objekt zugeordnete Objekt-Menge zurck. Wenn die spezifizierte Objekt-Menge nicht online nderbar ist, dann untersttzt
	 * die zurckgegebene Menge die {@link NonMutableSet Schnittstelle fr nicht online nderbare Mengen}. Wenn die Menge online nderbar ist, dann untersttzt das
	 * zurckgegebene Mengenobjekt die {@link MutableSet Schnittstelle fr online nderbare Mengen}.
	 *
	 * @param name Der Name der gewnschten Menge
	 *
	 * @return Menge von System-Objekten oder <code>null</code>, wenn die spezifizierte Menge nicht vorhanden ist.
	 */
	public ObjectSet getObjectSet(String name);

	/**
	 * Liefert die Liste aller diesem Konfigurationsobjekt zugeordneten Mengen zurck.
	 *
	 * @return Liste aller Mengen dieses Objekts. Die in der Liste enthaltenen Mengen implementieren je nach Art entweder die {@link NonMutableSet Schnittstelle
	 *         fr nicht online nderbare Mengen} oder die {@link MutableSet Schnittstelle fr online nderbare Mengen}.
	 */
	public List<ObjectSet> getObjectSets();

	/**
	 * Macht ein bereits als ungltig markiertes Objekt wieder gltig. Wenn ein Konfigurationsobjekt mit der Methode {@link SystemObject#invalidate} fr eine
	 * zuknftige Konfigurationsversion als ungltig markiert wurde und diese Konfigurationsversion noch nicht aktiviert wurde, dann kann das Objekt durch Aufruf
	 * dieser Methode wieder gltig gemacht werden.
	 *
	 * @throws ConfigurationChangeException Wenn das Objektes nicht wieder gltig gemacht werden konnte.
	 */
	public void revalidate() throws ConfigurationChangeException;

	/**
	 * Dupliziert ein Konfigurationsobjekt. Es ist zu beachten, dass Komponenten nicht isoliert dupliziert werden knnen, sondern im Sinne der Komposition immer
	 * nur ganze Objekt-Einheiten zusammen dupliziert werden knnen, d.h. ausgehend von einem freien Objekt wird das Objekt mit all seinen Komponenten rekursiv
	 * dupliziert. Das Duplikat wird mit Aktivierung der in Bearbeitung befindlichen neuen Version des jeweiligen Konfigurationsbereichs gltig. Da die Pids gleich
	 * bleiben, muss zuvor das "alte" Objekt {@link #invalidate() gelscht} werden.
	 *
	 * @return Das Duplikat dieses Konfigurationsobjekts.
	 *
	 * @throws ConfigurationChangeException Falls das Objekt kein freies Objekt ist und das Duplizieren nicht erlaubt ist oder das Duplikat nicht erstellt werden
	 *                                      konnte.
	 * @see #duplicate(java.util.Map)
	 */
	public SystemObject duplicate() throws ConfigurationChangeException;

	/**
	 * Dupliziert ein Konfigurationsobjekt. Es ist zu beachten, dass Komponenten nicht isoliert dupliziert werden knnen, sondern im Sinne der Komposition immer
	 * nur ganze Objekt-Einheiten zusammen dupliziert werden knnen, d.h. ausgehend von einem freien Objekt wird das Objekt mit all seinen Komponenten rekursiv
	 * dupliziert. Das Duplikat wird mit Aktivierung der in Bearbeitung befindlichen neuen Version des jeweiligen Konfigurationsbereichs gltig.
	 * <p/>
	 * Zu ersetzende Pids der Komponenten knnen in der Map (altePid, neuePid) bergeben werden. Nicht ersetzte Pids, deren Objekte aber durch Komposition an die
	 * Objekt-Einheit gebunden sind, mssen vor Aktivierung {@link #invalidate() gelscht} werden.
	 *
	 * @param substitudePids Map, die die Wert-Paare (altePid, neuePid) enthlt.
	 *
	 * @return Das Duplikat dieses Konfigurationsobjekts.
	 *
	 * @throws ConfigurationChangeException Falls das Objekt kein freies Objekt ist und das Duplizieren nicht erlaubt ist oder das Duplikat nicht erstellt werden
	 *                                      konnte.
	 * @see #duplicate()
	 */
	public SystemObject duplicate(Map<String, String> substitudePids) throws ConfigurationChangeException;

	/**
	 * Ordnet dem Konfigurationsobjekt eine weitere Menge zu. Die Zuordnung wird erst mit der nchsten Konfigurationsversion gltig.
	 *
	 * @param set Menge, die dem Konfigurationsobjekt zugeordnet werden soll.
	 *
	 * @throws ConfigurationChangeException Wenn die Menge nicht hinzugefgt werden konnte.
	 */
	public void addSet(ObjectSet set) throws ConfigurationChangeException;

	/**
	 * Entfernt die Zuordnung von diesem Konfigurationsobjekt zu einer Menge. Die nderung wird erst mit der nchsten Konfigurationsversion gltig.
	 *
	 * @param set Menge, die entfernt werden soll.
	 *
	 * @throws ConfigurationChangeException Wenn die Menge nicht entfernt werden konnte.
	 */
	public void removeSet(ObjectSet set) throws ConfigurationChangeException;
}

